/*-
 * Copyright (c) 2000 Murray Stokely
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#ifdef HAVE_LIBUTIL_PROPERTY
#include <libutil.h>
#else
#include "property.h"
#endif

#define CLIENT_PORT 31337

typedef struct s_conf {
    char hostname[256];
    int port;
} conf_t;


int
readConfig(conf_t *conf)
{
    int fd;
    properties propList;
    char *value = NULL;

    fd = open(".updatec",O_RDONLY);
    if (fd < 0)
	return -1;

    propList = properties_read(fd);
    if (propList == NULL) 
	return -1;

    value = property_find(propList,"server");
    if (value == NULL) {
	printf("server hostname not defined!\n");
	return -1;
    } else {
	strlcpy(&conf->hostname[0],value,256);
    }

    value = property_find(propList, "port");
    if (value == NULL) {
	printf("No port number, using %d!\n",CLIENT_PORT);
	conf->port = CLIENT_PORT;
    } else {
	conf->port = atoi(value);
    }

    return 0;
}

int
configClient(conf_t *conf)
{
    char *answer;
    int num;
    char prompt[256];


    answer = readline("Please enter the hostname or IP of the update server: ");

    strlcpy(&conf->hostname[0], answer, 256);

    snprintf(prompt,256,"What port should we use [%d]: ", CLIENT_PORT);
    answer = readline(prompt);
    if (answer)
	conf->port = atoi(answer);
    else
	conf->port = CLIENT_PORT;

    snprintf(prompt,256,"Would you like to save this configuration? [y/N] : ");
    answer = readline(prompt);
    if ((answer[0] == 'Y') || (answer[0] == 'y'))
	writeConfig(conf);

    return 0;
}

int
writeConfig(conf_t *conf)
{
  FILE *fp;
  fp = fopen(".updatec", "w");
  if (!fp)
    return -1;
  fprintf(fp,"server = %s\n",conf->hostname);
  fprintf(fp,"port = %d\n",conf->port);
  fclose(fp);
  return 0;
}

/* Input sanity checking routines */
