#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/uio.h>

#include "wrappers.h"
#include "netops.h"
#include "systool.h"
#include "filetree.h"
#include "commands.h"


/* installtree()
 *
 * Install trees of files based on a linked list of lines. This function is
 * alarmingly huge and should deffinitly be divided up someday.
 */
/* TODO: HANDLE 'P' FOR PACKAGES AND 'A' FOR ACTIONS, SPACES */
int
installtree(int sock, char *profile, char *dist, struct linelist *entries, int logfd, int level)
{
    char backup[FILENAME_MAX];
    char **args;
    char *info, *message;
    int count, space, err, levelerr;
    mode_t mode;
    uid_t uid;
    gid_t gid;
    struct linelist *indist;


    /* figure out what directory to store backups in */
    if(getenv(ENV_BACKUP))
	strlcpy(backup, getenv(ENV_BACKUP), sizeof(backup));
    else
	strlcpy(backup, DEF_BACKUP, sizeof(backup));

    /* specify modes entirely on our own */
    umask(0);

    levelerr = 0;

    /* run through all lines passed to the function */
    while(entries->next != NULL)
    {
	args = getArgs(entries->line, &count);
	err = 0;

	if(strcmp(*(args), DISTFLAG) == 0)
	{
	    /* entry is a distribution */

	    /* move to next entry if this is up to date */
	    if(!newversion(NULL, *(args+1), *(args+2)))
		goto next;

	    /* ask server for the distribution's contents */
	    info = Malloc(strlen(CMD_GETDIST) + strlen(profile) + strlen(*(args+1)) + 4);
	    sprintf(info, "%s %s %s\n", CMD_GETDIST, profile, *(args+1));
	    sendmessage(sock, info);
	    indist = recvmessage(sock);

	    /* leave a message before moving into the distribution */
	    space = level;
	    while(space--)
		write(logfd, " ", 1);
	    asprintf(&message, "distribution %s\n", *(args+1));
	    write(logfd, message, strlen(message));
	    free(message);

	    /* grab the components of the distribution */
	    if(!installtree(sock, profile, *(args+1), indist, logfd, level+1))
	    {
		setversion(NULL, *(args+1), *(args+2));
		asprintf(&message, "%s %s\n", *(args+1), LOGOK);
	    }
	    else
	    {
		asprintf(&message, "%s %s\n", *(args+1), LOGFAIL);
		levelerr = 1;
	    }

	    /* log success of the whole distribution */
	    space = level;
	    while(space--)
		write(logfd, " ", 1);

	    write(logfd, message, strlen(message));
	    free(message);
		
	    /* clean up */
	    freelines(indist);
	    free(info);
	}

	else if(strcmp(*(args), FILEFLAG) == 0)
	{
	    /* entry is a file or directory */

	    /* move to next entry if this is up to date */
	    if(!newversion(dist, *(args+1), *(args+5)))
		goto next;

	    info = Malloc(strlen(backup) + strlen(*(args+1)) + 1);
	    strlcpy(info, backup, strlen(backup) + 1);
	    strlcat(info, *(args+1), strlen(backup) + strlen(*(args+1)) + 1);
	    mode = atoi(*(args+2));
	    uid = atoi(*(args+3));
	    gid = atoi(*(args+4));

	    if(*(info+strlen(info)-1) == '/')
	    {
		/* entry is a directory, set it and a backup version up */
		*(info+strlen(info)-1) = '\0';
		if(mkdir(info, mode))
		{
		    if(errno != EEXIST)  /* no problem if directory exists */
			err = 1;
		    else
			if( 0 /*chmod(info, mode) XXX allow non-root to run */)
			    err = 1;
		}
		if(chown(info, uid, gid))
		    err = 1;

		/* disturbingly ugly but it works */
		strcpy(info, (info+strlen(backup)));
		if(mkdir(info, mode))
		{
		    if(errno != EEXIST)
			err = 1;
		    else
			if(0 /*chmod(info, mode) XXX allow non-root to run */)
			    err = 1;
		}
		if(chown(info, uid, gid))
		    err = 1;

		/* log success or failure, indented to represent tree depth */
		space = level;
		while(space--)
		    write(logfd, " ", 1);

		if(!err)
		{
		    asprintf(&message, "%s %s\n", info, LOGOK);
		    setversion(dist, *(args+1), *(args+5));
		}
		else
		{
		    asprintf(&message, "%s %s\n", info, LOGFAIL);
		    levelerr = 1;
		}

		write(logfd, message, strlen(message));
		free(message);
	    }
	    else
	    {
		/* must be a file */
		if(installfile(sock, profile, dist, *(args+1), backup, mode))
		    err = 1;
		if(chown(info, uid, gid))
		    err = 1;

		/* log with significant indentation */
		space = level;
		while(space--)
		    write(logfd, " ", 1);

		if(!err)
		{
		    asprintf(&message, "%s %s\n", *(args+1), LOGOK);
		    setversion(dist, *(args+1), *(args+5));
		}
		else
		{
		    asprintf(&message, "%s %s\n", *(args+1), LOGFAIL);
		    levelerr = 1;
		}

		write(logfd, message, strlen(message));
		free(message);
	    }

	    /* done with entry, clean up */
	    free(info);
	}

    next:
	free(args);
	entries = entries->next;
    }

    return(levelerr);
}


/* newversion()
 *
 * Compare version of given entry to that of the installed entry, and return
 * 1 if the given entry should be installed.
 */
/* TODO: USE DIST TO CHECK IF PACKAGE, CHECK VERSION APPROPRIATLY */
int
newversion(char *dist, char *entry, char *version)
{
    FILE *verfd;
    char verfile[FILENAME_MAX], temp[FILENAME_MAX];
    char **args;
    int count;


    /* obtain the proper directory */
    if(getenv(ENV_VERDIR))
	strlcpy(verfile, getenv(ENV_VERDIR), sizeof(verfile));
    else
	strlcpy(verfile, DEF_VERDIR, sizeof(verfile));

    /* finish of the full path */
    strlcat(verfile, "/", sizeof(verfile));
    if(!dist)
	strlcat(verfile, "distributions", sizeof(verfile));
    else
	strlcat(verfile, dist, sizeof(verfile));

    /* no version recorded, assume the requested version is newer */
    if((verfd = fopen(verfile, "r")) == NULL)
	return(1);

    /* look through the file for an appropriate entry */
    while(fgets(temp, sizeof(temp), verfd) > 0)
    {
	args = getArgs(temp, &count);

	if(count > 1)
	    if(!strncmp(*(args), entry, strlen(entry)))
	    {
		if(strncmp(*(args+1), version, strlen(version)) > 0)
		{
		    fclose(verfd);
		    return(0);
		}
		else
		{
		    fclose(verfd);
		    return(1);
		}
	    }
	free(args);
    }

    /* did not find it, clean up and assume the requested version is newer */
    fclose(verfd);
    return(1);
}


/* setversion()
 *
 * Update the version listing of the given entry.
 */
/* TODO: PROPER HANDLING OF PACKAGES */
void
setversion(char *dist, char *entry, char *version)
{
    FILE *verfd, *outfd;
    char verfile[FILENAME_MAX], outfile[FILENAME_MAX], temp[FILENAME_MAX];
    char **args;
    int count;


    /* obtain the proper directory */
    if(getenv(ENV_VERDIR))
	strlcpy(verfile, getenv(ENV_VERDIR), sizeof(verfile));
    else
	strlcpy(verfile, DEF_VERDIR, sizeof(verfile));

    /* finish off the file names */
    strlcat(verfile, "/", sizeof(verfile));
    strlcpy(outfile, verfile, sizeof(outfile));
    strlcat(outfile, "temp", sizeof(outfile));
    if(!dist)
	strlcat(verfile, "distributions", sizeof(verfile));
    else
	strlcat(verfile, dist, sizeof(verfile));

    /* open up the files */
    if(!(outfd = fopen(outfile, "w")))
	return;


    if((verfd = fopen(verfile, "r")))
    {
        while(fgets(temp, sizeof(temp), verfd) > 0)
        {
	    args = getArgs(temp, &count);

	    if(count > 1)
	        if(strncmp((*args), entry, strlen(entry)))
		    fprintf(outfd, "%s %s", *(args), *(args+1));

            free(args);
        }
	fclose(verfd);
    }

    fprintf(outfd, "%s %s\n", entry, version);
    fclose(outfd);

    snprintf(temp, sizeof(temp), "mv %s %s", outfile, verfile);
    system(temp);

    return;
}


/* installfile()
 *
 * Ask the server for a file, read it from the socket, backup whatever used to
 * be in it's place, and write it out to the disk.
 * 
 */
/* TODO: TIMEOUT ON TRANSFERS? */
int
installfile(int sock, char *profile, char *dist, char *file, char *backup, int mode)
{
    char buff[BUFFSIZE];
    char *newf;
    int n, got, left;
    int outfile;


    /* XXX prepend /usr/new to file, save to newf, and write to newf for the
     * sake of sanity while testing
     */
    asprintf(&newf, "/usr/new%s", file);

    /* ask server for the file */
    snprintf(buff, BUFFSIZE, "%s %s %s %s\n", CMD_GETFILE, profile, dist, file);
    sendmessage(sock, buff);

    /* first SIZELEN bytes give the length */
    readn(sock, buff, SIZELEN);
    left = atoi(buff);

    /* save the old file if necessary and create the new one */
    /* XXX this has GOT to be done with internal code in the near future */
    snprintf(buff, BUFFSIZE, "test -f %s", newf);
    if(!system(buff))
    {
	snprintf(buff, BUFFSIZE, "mv %s %s/%s", newf, backup, file);
	if(system(buff))
	    return(-1); /* XXX mem leak on newf! (temp while testing) */
    }

    if((outfile = open(newf, O_CREAT | O_RDWR, mode)) == -1)
	return(-1);

    free(newf); /* XXX (once again, only while testing) */

    /* read buffers from the socket and write them to the file */
    while(left)
    {
	if(left < BUFFSIZE)
	    n = left;
	else
	    n = BUFFSIZE;

	got = Read(sock, buff, n);

	left -= got;
 	write(outfile, buff, got);
    }

    /* ditch the EOF and finish up */
    Read(sock, buff, 1);
    close(outfile);

    return(0);
}
