/*-
 * Copyright (c) 2000 Murray Stokely, BSDi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Tcl bindings for the client communication code 'updates' package.
 *
 */

#include <tcl.h>
#include "systool.h"
#include "netops.h"

int 
open_connection(ClientData clientData,
		    Tcl_Interp *interp,
		    int argc, char *argv[]);

int Updates_Init(Tcl_Interp *interp) {
    Tcl_CreateCommand(interp, "open_connection", open_connection,
		      (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
    
    /* Declare that we implement the updates package so scripts can do
       a "package require updates" to load the library automatically
    */
    Tcl_PkgProvide(interp, "updates", "1.0");
    return TCL_OK;
}

/* Open connection username password */

int 
open_connection(ClientData clientData,
		    Tcl_Interp *interp,
		    int argc, char *argv[]) {
    int i, error, retval;
    int port = 31337;

    if (argc > 5) {
	interp->result = "ERROR: usage open_connection server port username password";
	return TCL_ERROR;
    }
    if (argc > 4) {
	error = Tcl_GetInt(interp, argv[2], &port);
	if (error != TCL_OK) 
	    return error;
    }

    retval = servattach(argv[1], port, argv[3], argv[4]);
    if (retval > 0)
	sprintf(interp->result, "%d", retval);
    else
	switch (retval) {
	case FAILROOT:
	    sprintf(interp->result, "ERROR: FAILROOT", retval);
	    break;
	case FAILLOOKUP:
	    sprintf(interp->result, "ERROR: Could not lookup hostname!", retval);
	    break;
	case FAILSOCK:
	    sprintf(interp->result, "ERROR: Could not create socket!", retval);
	    break;
	case FAILCONN:
	    sprintf(interp->result, "ERROR: Could not create a connection to the server!", retval);
	    break;
	case FAILAUTH:
	    sprintf(interp->result, "ERROR: Authorization Failed", retval);
	    break;
	default:
	    sprintf(interp->result, "ERROR: Unknown (%d)", retval);
	    break;
	}
    return TCL_OK;
}
