/*
 * Most of these functions are derived from 
 *
 *    Unix Network Programming Volume 1, 2nd Edition
 *    W. Richard Stevens
 *
 * Any mistakes are most likely my own, not the authors
 * - Murray Stokely
 */

#define MAXLINE 256
#define SIZELEN 20
/* Turn on the memory checker */
/* define CHECKMEM */

#define WriteX(fd, msg) Write(fd, msg, strlen(msg))
#define Malloc(size) My_Malloc(size, __FILE__, __LINE__)
#define Free(ptr) My_Free(ptr, __FILE__, __LINE__)

#ifdef CHECKMEM
typedef struct s_leak {
    char malloc_filename[256];
    int malloc_linenum;
    char free_filename[256];
    int free_linenum;
    void *ptr;
    struct s_leak *next;
} leak_t;
#endif

/* Prototypes for the wrapper functions */
ssize_t Readline(int fd, void *vptr, size_t maxlen);
int SocketW(int family,int type,int protocol);
void Bind(int fd, const struct sockaddr *sa, socklen_t salen);
int Accept(int fd, struct sockaddr *sa, socklen_t *salenptr);
int Connect(int fd, const struct sockaddr *sa, socklen_t salen);
void Close(int fd);
void Listen(int fd, int backlog);
ssize_t Read(int fd, void *ptr, size_t nbytes);
void Write(int fd, void *ptr, size_t nbytes);
int Open(const char *pathname, int oflag, mode_t mode);
void Exit(char *msg);
void *My_Malloc(size_t size, char *filename, int linenum);
void My_Free(void *ptr,  char *filename, int linenum);
pid_t Fork(void);

char **getArgs(char *command, int *n);
void chomp(char **string);
/* readn() is also usefull outside of the file it lives in */
ssize_t readn(int fd, void *vptr, size_t n);
