#!/usr/bin/perl
# ----------------------------------------------------------------------------
# Populate
#
# This script takes a release and inputs it into the database.
# You should run this from the <CHROOTDIR>/R/stage/trees
# where <CHROOTDIR> is the directory created by a make release.
# ----------------------------------------------------------------------------
#
#
#~/updated/server/populate.pl -release 4.2-RELEASE -reltag 1 > sql.out
#
# and then 'cat sql.out | mysql -u updates updates'
#

use Getopt::Long;

sub printHelp() {
    print "This is the help function\n";
}

&GetOptions("help", "release=s", "reltag=i");

if (($opt_help) || (!$opt_release) || (!$opt_reltag)) {
    printHelp();
    exit();
}

sub getTime($) {
    my $seconds = $_[0];
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime($seconds);
    sprintf "%04d-%02d-%02d %02d:%02d:%02d",$year + 1900,$mon,$mday,$hour,$min,$sec;
}

printf "INSERT into profile values ('4.2-STANDARD', '$opt_reltag', 'A stock FreeBSD-4.2 system');\n";
printf "INSERT into releases values ('$opt_reltag', '$opt_release', NULL);\n";

$cwd = `pwd`;
chomp $cwd;
my $mtreedir = "bin/etc/mtree";

## First we should get the mtree files
opendir(DIR, $mtreedir);
@mtreelist = readdir(DIR);
closedir(DIR);
chdir $mtreedir;

foreach $file (@mtreelist) {
    my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, 
	$atime, $mtime, $ctime, $blksize, $blocks) = stat $file;
    my $filetime = getTime($mtime);
    my $component = 0;
    if (-f $file) {
	if ($file =~ /BSD.x11-4.dist/) {
	    $component = 2; # XXX X4 is hardcoded as component 2
	} elsif ($file =~ /BSD.x11.dist/) {
	    $component = 1; # XXX X is hardcoded as component 1
	} else {
	    $component = 0;
	}
	printf "INSERT into dirs values ('$opt_reltag', $component, '%s', '$cwd/$mtreedir/$file');\n", $filetime;
    }
}
chdir $cwd;

opendir(DIR, ".");
@filelist = readdir(DIR);
closedir(DIR);

foreach $file (@filelist) {
    if (-d $file) {
	next if ($file eq '.');
	next if ($file eq '..');
	print STDERR "Looking through $file\n";
	# This is a directory (a distribution)
	my $disttime = 0;

	chdir($file);
	@files = `find .`;
	foreach (@files) {
	    s|^\.||;
	    chomp;
	    my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, 
		$atime, $mtime, $ctime, $blksize, $blocks) = stat $_;
	    my $filetime = getTime($mtime);
	    if (-f) {
		printf "INSERT into files values ('$opt_reltag', '%s', '$file', '$_', '$cwd/$file', %d, %d, %d);\n", $filetime, $mode, $uid, $gid;
	    }
# elsif (-d)
#		printf "Assume this is a directoryprintf "INSERT into dirs values ('$opt_reltag', '%s', '$file', '$_', %d, %d, %d);\n",$filetime, $mode, $uid, $gid;
#	    } elsif (-f) {
#
#	    } elsif (-b) {
#		printf "INSERT into devs values ('$opt_reltag', '%s', '$file', '$_', 1, %d, 0, 0, %d, %d);\n", $filetime, $mode, $uid, $gid;
#	    } elsif (-c) {
#		printf "INSERT into devs values ('$opt_reltag', '%s', '$file', '$_', 0, %d, 0, 0, %d, %d);\n", $filetime, $mode, $uid, $gid;
#	    } else {
#		printf STDERR "Don't know how to deal with '$_'\n";
#	    }
	    $disttime = $filetime if ($filetime > $disttime);
	}
	printf "INSERT into dists values ('$opt_reltag', '%s', '$file');\n", $disttime;

	chdir($cwd);
    }
}

