/*-
 * Copyright (c) 2000 Murray Stokely, BSDi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "updated.h"
#include "wrappers.h"

void 
listProfiles(int fd, db_t *db) {
    resultset_t *results;
    resultrow_t *resultrow;
    char *profile;

    /* Connect to the database */
    if ((db->connect)(db)) {
      printf("Error! could not connect to database!\n");
      Write(fd,"ERROR\n",strlen("ERROR\n"));
      return;
    }

    results = (db->select)(db, "select * from profile");
    if (results == NULL) {
	printf("select * from profile returned NULL!\n");
	WriteX(fd, "ERROR: NO PROFILES\n");
	return;
    } else {
	for (resultrow = results->rows;resultrow!=NULL;resultrow = resultrow->next) {
	    profile = sql_getvalue(resultrow,"name");
	    Write(fd,profile,strlen(profile));
	    Write(fd,"\n",1);
	}
    }
    sql_freeresults(results);
/*    ((db->disconnect)(db)); */
}

void
getProfile(int fd, db_t *db, char *profile)
{
    resultset_t *results;
    resultrow_t *resultrow;
    char *filename, *timestamp;
    char *query = Malloc(256);

    /* Connect to the database */
    if ((db->connect)(db)) {
	printf("Error! could not connect to database!\n");
	WriteX(fd, "ERROR: COULD NOT CONNECT TO DB\n");
	return;
    }

    /* XXX - MySQL specific query that gets the timestamp into numeric mode */
    snprintf(query, 256, "select dists.distribution, dists.timestamp + 0 as timestamp from dists,profile where dists.reltag = profile.reltag and profile.name = '%s'",profile);
    results = (db->select)(db, query);
    if (results == NULL) {
	WriteX(fd, "ERROR: NO SUCH PROFILE\n");
	return;
    } else {
	for (resultrow = results->rows;resultrow!=NULL;resultrow = resultrow->next) {
	    if (!(filename = sql_getvalue(resultrow,"distribution"))) {
		WriteX(fd, "ERROR: NULL DISTRIBUTION\n");
		return;
	    }
	    if (!(timestamp = sql_getvalue(resultrow, "timestamp"))) {
		WriteX(fd, "ERROR: NULL TIMESTAMP\n");
		return;
	    }
	    Write(fd, "D ", 2);
	    WriteX(fd, filename);
	    Write(fd, " ", 1);
	    WriteX(fd, timestamp);
	    Write(fd, "\n", 1);
	}
    }
    sql_freeresults(results);
    Free(query);
    /*    ((db->disconnect)(db));*/
}

/* Return a list of all files in the specified distribution and profile */

void
getDist(int fd, db_t *db, char *profile, char *dist) {
    resultset_t *results;
    resultrow_t *resultrow;
    char *filename, *uid, *gid, *mode, *timestamp;
    char *query = Malloc(256);

    /* Connect to the database */
    if ((db->connect)(db)) {
	WriteX(fd, "ERROR : Could not connect to database!\n");
	return;
    }

    /* Get the directories in this distribution */
    snprintf(query, 256, "select filename, timestamp + 0 as timestamp, mode, uid, gid from dirs,profile where dirs.distribution = '%s' and dirs.reltag = profile.reltag and profile.name = '%s'", dist, profile);

    results = (db->select)(db, query);
    if (results == NULL) {
	Write(fd, "ERROR: NO SUCH PROFILE\n", 23);
	/* Print out a SQL error here from the backend? */
	Free(query);
	return;
    } else {
	for (resultrow = results->rows;resultrow!=NULL;resultrow = resultrow->next) {
	    filename = sql_getvalue(resultrow,"filename");
	    mode = sql_getvalue(resultrow, "mode");
	    uid = sql_getvalue(resultrow, "uid");
	    gid = sql_getvalue(resultrow, "gid");
	    timestamp = sql_getvalue(resultrow, "timestamp");
	    if (!filename || !mode || !uid || !gid || !timestamp) {
		WriteX(fd, "ERROR: SQL\n");
		Free(query);
		return;
	    }
	    Write(fd, "F ", 2);
	    Write(fd, filename, strlen(filename));
	    Write(fd, "/ ", 2);
	    WriteX(fd, mode);
	    Write(fd, " ", 1);
	    WriteX(fd, uid);
	    Write(fd, " ", 1);
	    WriteX(fd, gid);
	    Write(fd, " ", 1);
	    WriteX(fd, timestamp);
	    Write(fd, "\n", 1);
	}
    }
    sql_freeresults(results);

    /* Now get the files */
    snprintf(query, 256, "select filename, timestamp + 0 as timestamp, mode, uid, gid from files,profile where files.distribution = '%s' and files.reltag = profile.reltag and profile.name = '%s'", dist, profile);
    results = (db->select)(db, query);
    if (results == NULL) {
	Write(fd, "ERROR: NO SUCH RELEASE\n", 23);
	Free(query);
	return;
    } else {
	for (resultrow = results->rows;resultrow!=NULL;resultrow = resultrow->next) {
	    filename = sql_getvalue(resultrow, "filename");
	    mode = sql_getvalue(resultrow, "mode");
	    uid = sql_getvalue(resultrow, "uid");
	    gid = sql_getvalue(resultrow, "gid");
            timestamp = sql_getvalue(resultrow, "timestamp");
	    if (!filename || !mode || !uid || !gid || !timestamp) {
		WriteX(fd, "ERROR: SQL\n");
		sql_freeresults(results);
		Free(query);
		return;
	    }
	    Write(fd, "F ", 2);
	    Write(fd, filename, strlen(filename));
	    Write(fd, " ", 1);
	    WriteX(fd, mode);
	    Write(fd, " ", 1);
	    WriteX(fd, uid);
	    Write(fd, " ", 1);
	    WriteX(fd, gid);
	    Write(fd, " ", 1);
	    WriteX(fd, timestamp);
	    Write(fd, "\n", 1);
	}
    }
    sql_freeresults(results);
    Free(query);
    /*    ((db->disconnect)(db));	*/
}

void
getFile(int connfd, db_t *db, char *profile, char *dist, char *filename) {
    int fd;
    char *query;
    resultset_t *results;
    resultrow_t *resultrow;
    char *location, *fullpath;
    off_t numwritten;
    struct stat sb;
    char buf[2048];
    query = Malloc(4096);
    fullpath = Malloc(1024);

    /* Connect to the database */
    if ((db->connect)(db)) {
	WriteX(connfd, "ERROR : could not connect to database!\n");
	return;
    }

    /* XXX What if fileames or releases have special chars?  escape! */
    snprintf(query, 4096, "select files.location from files, profile where files.filename = '%s' and files.reltag = profile.reltag and profile.name = '%s'", filename, profile);

    results = (db->select)(db, query);

    if (results == NULL) {
	WriteX(connfd, "ERROR: SQL\n\n");
	Free(query);
	return;
    } else {
	resultrow = results->rows;
	if (!(location = sql_getvalue(resultrow,"location"))) {
	    WriteX(connfd, "ERROR: SQL\n\n");
	    sql_freeresults(results);
	    Free(query);
	    return;
	}
	SAFE_STRCPY(fullpath,location,256);
	strncat(fullpath,filename,256);
    }

    fd = open(fullpath, O_RDONLY);
    if (fd < 1)
	perror("open");

    if (fstat(fd, &sb))
	perror("stat");
    /* it looks like we have to hardcode the printf format specifier, yuck */
    snprintf(query, SIZELEN+1, "%20.0d", (unsigned long)sb.st_size);
    Write(connfd, query, SIZELEN);
/*	Readline(connfd, buf, 2048);*/
    if (sendfile(fd, connfd, 0, 0, NULL, &numwritten, 0)) 
	perror("sendfile");

/*    printf("We just sent %d bytes of %s\n", numwritten, fullpath);*/
    close(fd);
    sql_freeresults(results);
    Free(query);
    Free(fullpath);
    /*    ((db->disconnect)(db));*/
}
