/*-
 * Copyright (c) 2000 Murray Stokely
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>

#include "updated.h"
#include "wrappers.h"

int
main(int argc, char **argv)
{
    db_t db;
    int fd;
    char *line = Malloc(4096);

    printf("Binary Updater Database Table Setup Tool\n\n");
    printf("This program just sets up the tables, you must create the database.\n");

    if (readConfig(&db)) {
	printf("Manual configuration required!\n\n");
	if (configDB(&db)) {
	    printf("Error during manual configuration!\n");
	    return -1;
	}
    }

    if (initDB(&db))
	printf("Error initializing internal database structure!\n");

    /* Connect to the database */
    if ((db.connect)(&db)) {
	printf("Error! could not connect to database!\n");
	return -1;
    }

    if (db.software == T_MYSQL) {
	if ((fd = open("./setup-mysql.sql", O_RDONLY)) < 1)
	    Exit("Open setup-mysql.sql");
    } else if (db.software == T_PGSQL) {
	if ((fd = open("./setup-pgsql.sql", O_RDONLY)) < 1)
	    Exit("Open setup-pgsql.sql");
    }

    while (Readline(fd, line, 4096 -1 )) {
	if (line[0] == '#')
	    continue;
	chomp(&line);
	if ((db.execute)(&db, line)) {
	    printf("SQL Error! command: \n%s\n", line);
	    return -1;
	} else {
	    printf(".");
	    fflush(NULL);
	}
    }
    printf("done.\n");
    return 0;
}
