/*-
 * Copyright (c) 2000, 2001 Murray Stokely
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "updated.h"
#include "wrappers.h"


/* 
 * This function initializes the function pointers in our database
 * structure to the software specific functions
 */

int 
initDB(db_t *db)
{
    if (db->software == 0) {
	printf("No database software selected!\n");
	return -1;
    }

#if !defined(HAVE_MYSQL) && !defined(HAVE_PGSQL)
#error "You must use at least one database software!"
#endif

#ifdef HAVE_MYSQL
    else if (db->software == T_MYSQL) {
	db->connect    = (SQLProc *)my_mysql_connect;
	db->disconnect = (SQLProc *)my_mysql_disconnect;
	db->execute    = (ExecProc *)my_mysql_execute;
	db->select     = (SelectProc *)my_mysql_select;
    }
#endif
#ifdef HAVE_PGSQL
    else if (db->software == T_PGSQL) {
	db->connect    = (SQLProc *)pgsql_connect;
	db->disconnect = (SQLProc *)pgsql_disconnect;
	db->execute    = (ExecProc *)pgsql_execute;
	db->select     = (SelectProc *)pgsql_select;
    }
#endif
    else {
	printf("You selected a database software that I know nothing about! '%d'\n", db->software);
	return -1;
    }
    db->handle = NULL;
    db->flags = 0;
    return 0;
}

char *
sql_getvalue(resultrow_t *resultrow, char *colname)
{
    resultcell_t *resultcell;
    for (resultcell = resultrow->cells; resultcell!=NULL;resultcell=resultcell->next) {
	if (!strncmp(resultcell->name, colname, strlen(colname)))
	    return resultcell->value;
    }
    return NULL;
}

void
sql_freeresults(resultset_t *results)
{
    int i;
    if (results == NULL)
	return;
    for (i = 0; i < results->numfields; i++) {
	Free(results->fields[i]);
    }
    Free(results->fields);
    sql_freerow(results->rows);
    Free(results);
}

void
sql_freerow(resultrow_t *resultrow)
{
    if (resultrow == NULL)
	return;
    sql_freerow(resultrow->next);
    sql_freecells(resultrow->cells);
    Free(resultrow);
}

void
sql_freecells(resultcell_t *resultcell)
{
    if (resultcell == NULL)
	return;
    sql_freecells(resultcell->next);
    Free(resultcell->value);
    Free(resultcell);
}
